# -*- coding: utf-8 -*-
"""

@author: Martin Paul
"""

import os
import glob
import numpy as np
from astropy.io import fits
from scipy.optimize import curve_fit




###############################################################################




###############################################################################
# 1) Set the working directory:                                               #
###############################################################################


sourceDir = "....\\Sensor_data\\input\\" # data directory

if not os.path.isdir(sourceDir):
	raise NotADirectoryError(sourceDir)



###############################################################################
# 2)  Imports all ".txt" files in the working directory                       #
###############################################################################


hotpixels = np.loadtxt(sourceDir + "hotpixels_1920_1200.txt") # known hotpixels of the sensor
hotpixels = hotpixels.astype("int") 

roiRegion = np.genfromtxt(sourceDir + "roi_region_x_y.txt") # 25 pixels +/- of the manually determined laser center
roiRegion = roiRegion.astype("int")

x_start = roiRegion[0,0];
x_end = roiRegion[1,0];

y_start = roiRegion[0,1];
y_end = roiRegion[1,1];


###############################################################################
# 3) imports all ".fits" files from the working directory                     #
###############################################################################

sf = glob.glob(sourceDir + "/*.fits")

print("In total " + str(len(sf)) + " .csv files were found and saved in the working dictionary!")
if not os.path.isdir(sourceDir):
	raise NotADirectoryError(sourceDir)



dataFrameDictCold = {} # removes known hotpixels of the sensor from the raw data

for i in range(len(sf)):
    interrim = fits.open(sf[i])[0].data
    for k in range(len(hotpixels)):
        #print("test")
        interrim[hotpixels[k][1],hotpixels[k][0]] = 0

    dataFrameDictCold[i] = interrim
    

###############################################################################
# 4) defines the gaussian two-dimensional Gaussian                           #
###############################################################################


# Two-dimensional Gaussian
def gaussian(x, y, x0, y0, xalpha, yalpha, A, offset):
    return A * np.exp( -((x-x0)/xalpha)**2 -((y-y0)/yalpha)**2)+offset


# Preperation for the fit * based on https://scipython.com/blog/non-linear-least-squares-fitting-of-a-two-dimensional-data/
def _gaussian(M, *args):
    x, y = M
    arr = np.zeros(x.shape)
    for i in range(len(args)//6):
       arr += gaussian(x, y, *args[i*6:i*6+6])
    return arr


x, y = np.arange(x_start,x_end,1) , np.arange(y_start,y_end,1)
X, Y = np.meshgrid(x, y)


###############################################################################
# 5) Dataevaluation over all frames                                           #
###############################################################################

#%%



# important starting parameters of the gaussian fit, the first two are the initial guess of the laser spot center based on the observation on the first frame
guess_prms = [(970 ,753 ,4 ,4 ,1000 , 0)]

evalType = "y"


if evalType == "y":

    results_array = []
    xdata = np.vstack((X.ravel(), Y.ravel()))


    for i in range(len(dataFrameDictCold)):
        
        roi_hw = 4 
        lb = 8
        hb = 3

        data_post = dataFrameDictCold[i][x_start:x_end,y_start:y_end]
        data_raw_median = np.median(data_post)
        cut_off = data_raw_median/1 
        data_post = data_post-cut_off
        data_post[data_post < 0] = 0
    
        popt, pcov = curve_fit(_gaussian, xdata, data_post.ravel(), guess_prms , maxfev=100000)
        fit = np.zeros(dataFrameDictCold[i][x_start:x_end,y_start:y_end].shape)
        for k in range(len(popt)//6):
            fit += gaussian(X, Y, *popt[k*6:k*6+6])

        x_center = int(round(popt[0]))-2
        y_center = int(round(popt[1]))+3
        

        interrim = np.mean(np.sort(dataFrameDictCold[i][x_center-roi_hw:x_center+roi_hw,y_center-roi_hw:y_center+roi_hw].flatten())[-lb:-hb])
        row =interrim
        fit = None

        results_array = np.append(results_array,[i,row,x_center,y_center])
        print("file number :" + str(i) + " x = : " + str(x_center) + " y = " + str(y_center) + " median : " + str(interrim))

    results_array = results_array.reshape(len(dataFrameDictCold),4)
else:
    print("error")


###############################################################################
# 6) Data export as .txt                                                      #
###############################################################################

fmt = "\t".join(["%s"] + ["%10.3e"] * (results_array.shape[1]-1))
np.savetxt("results" + ".txt", results_array, fmt=fmt, header=  # set here name for the export ".txt" file
"source directory of the raw data: " +"\n"+
"\n"+
"     " +     sourceDir + "\n" +
"\n"+
"\n"+
"fit guess: "  "\n" +
"\n"+
"     x0_guess = " + str(guess_prms[0][0]) + "\n"+
"     y0_guess = " + str(guess_prms[0][1]) + "\n"+
"     x_alpha guess = " + str(guess_prms[0][2]) + "\n"+
"     y_alpha guess = " + str(guess_prms[0][3]) + "\n"+
"     A guess = " + str(guess_prms[0][4]) + "\n"+
"     offset_guess = " + str(guess_prms[0][5]) + "\n"+
"\n"+
"\n"+
"Evaluation specific data: "  "\n" +
"\n"+
"     halfwidth of the ROI square in pixels    = " + str(roi_hw) + "\n"+
"     ROI square in pixels (number)   = " + str((roi_hw*2)*(roi_hw*2)) + "\n"+
"     roi_region (x) :" + str(roiRegion[0,0]) +  "-" + str(roiRegion[1,0]) + "\n"+
"     roi_region (Y) :" + str(roiRegion[0,1]) +  "-" + str(roiRegion[1,1]) + "\n"+
"     lower border pixel :" + str(lb) +  "\n"+
"     higher border pixel :" + str(hb) +  "\n"+
"\n"+
"Columns: "  "\n" +
"     1: frame number " + "\n" +
"     2: median intensity of the ROI" "\n" +
"     3: x0 center of the roi " "\n" +
"     4: y0_center of the roi" "\n" 
           , comments='')














